@@ -1,4 +1,4 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright IBM Corp. 2019, 2025
 // SPDX-License-Identifier: MPL-2.0
 
 package api
@@ -257,6 +257,10 @@ type ContainerSecurityConfig struct {
 	// If set to private, IDMappings must be set.
 	// Mandatory.
 	UserNS Namespace `json:"userns,omitempty"`
+	// IDMappings are UID and GID mappings that will be used by user
+	// namespaces.
+	// Required if UserNS is private.
+	IDMappings *IDMappingOptions `json:"idmappings,omitempty"`
 	// Groups are a list of supplemental groups the container's user will
 	// be granted access to.
 	// Optional.
@@ -312,12 +316,7 @@ type ContainerSecurityConfig struct {
 	// If ALL is passed, all paths will be unmasked.
 	// Optional.
 	Unmask []string `json:"unmask,omitempty"`
-	// IDMappings are UID and GID mappings that will be used by user
-	// namespaces.
-	// Required if UserNS is private.
-	// IDMappings *storage.IDMappingOptions `json:"idmappings,omitempty"`
 	// ReadOnlyFilesystem indicates that everything will be mounted
-
 	// as read-only
 	ReadOnlyFilesystem bool `json:"read_only_filesystem,omitempty"`
 }
@@ -372,8 +371,8 @@ type ContainerNetworkConfig struct {
 	// Only available if NetNS is set to bridge.
 	// Optional.
 	StaticMAC *net.HardwareAddr `json:"static_mac,omitempty"`
-	// PortBindings is a set of ports to map into the container.
-	// Only available if NetNS is set to bridge or slirp.
+	// PortMappings is a set of ports to map into the container.
+	// Only available if NetNS is set to bridge, pasta or slirp.
 	// Optional.
 	PortMappings []PortMapping `json:"portmappings,omitempty"`
 	// Expose is a number of ports that will be forwarded to the container
@@ -382,7 +381,7 @@ type ContainerNetworkConfig struct {
 	// protocol. Allowed protocols are "tcp", "udp", and "sctp", or some
 	// combination of the three separated by commas.
 	// If protocol is set to "" we will assume TCP.
-	// Only available if NetNS is set to Bridge or Slirp, and
+	// Only available if NetNS is set to Bridge, Pasta or Slirp, and
 	// PublishExposedPorts is set.
 	// Optional.
 	Expose map[uint16]string `json:"expose,omitempty"`
@@ -423,7 +422,7 @@ type ContainerNetworkConfig struct {
 	// random unused ports (guaranteed to be above 1024) on the host.
 	// This is based on ports set in Expose below, and any ports specified
 	// by the Image (if one is given).
-	// Only available if NetNS is set to Bridge or Slirp.
+	// Only available if NetNS is set to Bridge, Pasta or Slirp.
 	PublishExposedPorts bool `json:"publish_image_ports,omitempty"`
 	// UseImageResolvConf indicates that resolv.conf should not be managed
 	// by Podman, but instead sourced from the image.
@@ -569,6 +568,9 @@ const (
 	// Slirp indicates that a slirp4netns network stack should
 	// be used
 	Slirp NamespaceMode = "slirp4netns"
+	// Pasta indicates that a pasta (successor to slirp4netns)
+	// network stack should be used
+	Pasta NamespaceMode = "pasta"
 	// KeepID indicates a user namespace to keep the owner uid inside
 	// of the namespace itself
 	KeepID NamespaceMode = "keep-id"
@@ -585,6 +587,53 @@ type Namespace struct {
 	Value  string        `json:"value,omitempty"`
 }
 
+// ------------------------------------------------------------------------------------
+// structs copied from https://pkg.go.dev/go.podman.io/storage/types#IDMappingOptions
+
+type IDMappingOptions struct {
+	// UIDMap and GIDMap are used for setting up a layer's root filesystem
+	// for use inside of a user namespace where ID mapping is being used.
+	// If HostUIDMapping/HostGIDMapping is true, no mapping of the
+	// respective type will be used.  Otherwise, if UIDMap and/or GIDMap
+	// contain at least one mapping, one or both will be used.  By default,
+	// if neither of those conditions apply, if the layer has a parent
+	// layer, the parent layer's mapping will be used, and if it does not
+	// have a parent layer, the mapping which was passed to the Store
+	// object when it was initialized will be used.
+	HostUIDMapping bool
+	HostGIDMapping bool
+	UIDMap         []IDMap
+	GIDMap         []IDMap
+	AutoUserNs     bool
+	AutoUserNsOpts AutoUserNsOptions
+}
+
+type AutoUserNsOptions struct {
+	// Size defines the size for the user namespace.  If it is set to a
+	// value bigger than 0, the user namespace will have exactly this size.
+	// If it is not set, some heuristics will be used to find its size.
+	Size uint32
+	// InitialSize defines the minimum size for the user namespace.
+	// The created user namespace will have at least this size.
+	InitialSize uint32
+	// PasswdFile to use if the container uses a volume.
+	PasswdFile string
+	// GroupFile to use if the container uses a volume.
+	GroupFile string
+	// AdditionalUIDMappings specified additional UID mappings to include in
+	// the generated user namespace.
+	AdditionalUIDMappings []IDMap
+	// AdditionalGIDMappings specified additional GID mappings to include in
+	// the generated user namespace.
+	AdditionalGIDMappings []IDMap
+}
+
+type IDMap struct {
+	ContainerID int `json:"container_id"`
+	HostID      int `json:"host_id"`
+	Size        int `json:"size"`
+}
+
 // -------------------------------------------------------------------------------------------------------
 // structs copied from https://github.com/containers/podman/blob/master/libpod/define/container_inspect.go
 //
@@ -1483,6 +1532,7 @@ type HostInfo struct {
 	RemoteSocket   *RemoteSocket          `json:"remoteSocket,omitempty"`
 	Security       SecurityInfo           `json:"security"`
 	Slirp4NetNS    SlirpInfo              `json:"slirp4netns,omitempty"`
+	Pasta          PastaInfo              `json:"pasta,omitempty"`
 	RuntimeInfo    map[string]interface{} `json:"runtimeInfo,omitempty"`
 	Arch           string                 `json:"arch"`
 	BuildahVersion string                 `json:"buildahVersion"`
@@ -1515,6 +1565,14 @@ type SlirpInfo struct {
 	Version    string `json:"version"`
 }
 
+// PastaInfo describes the pasta executable that
+// is being being used.
+type PastaInfo struct {
+	Executable string `json:"executable"`
+	Package    string `json:"package"`
+	Version    string `json:"version"`
+}
+
 // IDMappings describe the GID and UID mappings
 // type IDMappings struct {
 // 	GIDMap []idtools.IDMap `json:"gidmap"`
